'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _koa = require('koa');

var _koa2 = _interopRequireDefault(_koa);

var _koaJson = require('koa-json');

var _koaJson2 = _interopRequireDefault(_koaJson);

var _koaBodyparser = require('koa-bodyparser');

var _koaBodyparser2 = _interopRequireDefault(_koaBodyparser);

var _koaQs = require('koa-qs');

var _koaQs2 = _interopRequireDefault(_koaQs);

var _koaMorgan = require('koa-morgan');

var _koaMorgan2 = _interopRequireDefault(_koaMorgan);

var _koaStatic = require('koa-static');

var _koaStatic2 = _interopRequireDefault(_koaStatic);

var _koa2Cors = require('koa2-cors');

var _koa2Cors2 = _interopRequireDefault(_koa2Cors);

var _koaPug = require('koa-pug');

var _koaPug2 = _interopRequireDefault(_koaPug);

var _koaSession = require('koa-session');

var _koaSession2 = _interopRequireDefault(_koaSession);

var _fileStreamRotator = require('file-stream-rotator');

var _fileStreamRotator2 = _interopRequireDefault(_fileStreamRotator);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _setupRoutes = require('./util/setup-routes');

var _setupRoutes2 = _interopRequireDefault(_setupRoutes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const cwd = process.cwd();
// import convert from 'koa-convert';

const {
  DOCUMENT_ROOT,
  LOG_ROOT,
  JSON_PRETTY,
  PUG_ENABLE,
  PUG_PAGES_PATH,
  STATIC_ROOT,
  CORS_ENABLE,
  CORS_ORIGIN,
  CORS_ALLOW_METHODS,
  SESSION_ENABLE,
  JSON_LIMIT = '1mb',
  FORM_LIMIT = '56kb'
} = process.env;

// 创建日志目录
const logDir = (0, _path.join)(cwd, LOG_ROOT);
_mkdirp2.default.sync(logDir);

// create a rotating write stream
const accessLogStream = _fileStreamRotator2.default.getStream({
  date_format: 'YYYYMMDD',
  filename: `${logDir}/access__%DATE%.log`,
  frequency: 'daily',
  verbose: false
});

class Qails {
  constructor() {
    this.koa = (0, _koaQs2.default)(new _koa2.default());
    this.use((0, _koaMorgan2.default)('combined', { stream: accessLogStream }));
    this.use((0, _koaStatic2.default)((0, _path.join)(cwd, STATIC_ROOT || 'static')));
    if (CORS_ENABLE) {
      const corsOptions = {
        origin: CORS_ORIGIN,
        allowMethods: CORS_ALLOW_METHODS.split(',')
      };
      // 允许一个或多个指定某域名能访问
      if (CORS_ORIGIN && CORS_ORIGIN !== '*') {
        corsOptions.origin = ctx => {
          const headerOrigin = ctx.request.header.origin;
          const isValidate = CORS_ORIGIN.split(',').some(whitelist => new RegExp(whitelist).test(headerOrigin));
          return isValidate ? headerOrigin : false;
        };
      }
      this.use((0, _koa2Cors2.default)(corsOptions));
    }
    if (SESSION_ENABLE === 'true') {
      this.use((0, _koaSession2.default)({}, this.koa));
    }
    this.use((0, _koaBodyparser2.default)({
      jsonLimit: JSON_LIMIT,
      formLimit: FORM_LIMIT
    }));
    this.use((0, _koaJson2.default)({ pretty: JSON_PRETTY === 'true' }));

    if (PUG_ENABLE === 'true') {
      const pug = new _koaPug2.default({ viewPath: (0, _path.join)(cwd, PUG_PAGES_PATH) });
      pug.use(this.koa);
    }

    (0, _setupRoutes2.default)(this.koa, (0, _path.join)(cwd, DOCUMENT_ROOT, 'routes'));

    this.server = null;
  }

  use(...args) {
    this.koa.use(...args);
    return this;
  }

  listen(port, cb) {
    this.server = this.koa.listen(port, cb);
    return this.server;
  }
}
exports.default = Qails;