'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Collection = exports.Model = exports.knex = undefined;

var _knex = require('knex');

var _knex2 = _interopRequireDefault(_knex);

var _bookshelf = require('bookshelf');

var _bookshelf2 = _interopRequireDefault(_bookshelf);

var _bookshelfModelbase = require('bookshelf-modelbase');

var _bookshelfModelbase2 = _interopRequireDefault(_bookshelfModelbase);

var _bookshelfCascadeDelete = require('bookshelf-cascade-delete');

var _bookshelfCascadeDelete2 = _interopRequireDefault(_bookshelfCascadeDelete);

var _bookshelfMask = require('bookshelf-mask');

var _bookshelfMask2 = _interopRequireDefault(_bookshelfMask);

var _bookshelfUuid = require('bookshelf-uuid');

var _bookshelfUuid2 = _interopRequireDefault(_bookshelfUuid);

var _bookshelfJsonColumns = require('bookshelf-json-columns');

var _bookshelfJsonColumns2 = _interopRequireDefault(_bookshelfJsonColumns);

var _bookshelfParanoia = require('bookshelf-paranoia');

var _bookshelfParanoia2 = _interopRequireDefault(_bookshelfParanoia);

var _magicCase = require('./magic-case');

var _magicCase2 = _interopRequireDefault(_magicCase);

var _features = require('./features');

var _features2 = _interopRequireDefault(_features);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const {
  KNEX_CLIENT,
  MYSQL_HOST,
  MYSQL_USER,
  MYSQL_PASSWORD,
  MYSQL_DATABASE,
  MYSQL_PORT
} = process.env;

const knexInstance = (0, _knex2.default)({
  client: KNEX_CLIENT,
  connection: {
    host: MYSQL_HOST,
    user: MYSQL_USER,
    password: MYSQL_PASSWORD,
    database: MYSQL_DATABASE,
    port: MYSQL_PORT
  }
});

const base = (0, _bookshelf2.default)(knexInstance);

// 让 Model 具有自动注册到中央位置的功能
if (_features2.default.MODEL_REGISTRY) {
  base.plugin('registry');
}

// 让 Model 具有返回虚拟字段的功能
if (_features2.default.MODEL_VIRTUALS) {
  base.plugin('virtuals');
}

// 让 Model 调用 toJSON 方法时具有显示／隐藏某些字段的功能
if (_features2.default.MODEL_VISIBILITY) {
  base.plugin('visibility');
}

// 让 Model 具有时间戳、数据校验和部分CURD功能
if (_features2.default.MODEL_BASE) {
  base.plugin(_bookshelfModelbase2.default.pluggable);
}
// 让 Model 具有分页功能
if (_features2.default.MODEL_PAGINATION) {
  base.plugin('pagination');
}

// 让 Model 具有删除关联数据功能
if (_features2.default.MODEL_CASCADEDELETE) {
  base.plugin(_bookshelfCascadeDelete2.default);
}

// 让 Model 具有返回自定义字段的功能
if (_features2.default.MODEL_MASK) {
  base.plugin(_bookshelfMask2.default);
}

// 让 Model 具有自动生成UUID的功能
if (_features2.default.MODEL_UUID) {
  const options = _features2.default.MODEL_UUID === true ? null : _features2.default.MODEL_UUID;
  base.plugin(_bookshelfUuid2.default, options);
}

// 让 Model 具有自动存储序列化对象的能力
if (_features2.default.MODEL_JSONCOLUMNS) {
  base.plugin(_bookshelfJsonColumns2.default);
}

// 让 Model 具有自动转换对象 key 拼写的能力
if (_features2.default.MODEL_MAGICCASE) {
  base.plugin(_magicCase2.default);
}

// 让 Model 具有软删除记录的能力
if (_features2.default.MODEL_SOFTDELETE) {
  const options = _features2.default.MODEL_SOFTDELETE === true ? null : _features2.default.MODEL_SOFTDELETE;
  base.plugin(_bookshelfParanoia2.default, options);
}

// 外部可以base.knex取到knex client
exports.default = base;
const knex = exports.knex = knexInstance;
const Model = exports.Model = base.Model;
const Collection = exports.Collection = base.Collection;