'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.snake = undefined;

var _lodash = require('lodash');

/**
 * 将 object-key 转为大写
 * @param {object|array|string} attributes
 * @param {array} excludeKeys
 * @return {object}
 */
function toCamelCase(attributes, excludeKeys = []) {
  const source = attributes;
  Object.keys(source).forEach(attribute => {
    const newAttribute = (0, _lodash.camelCase)(attribute);
    // json串内部的对象不做处理
    if ((0, _lodash.isObject)(source[attribute]) && excludeKeys.indexOf(attribute) === -1) {
      source[newAttribute] = toCamelCase(source[attribute], excludeKeys);
    } else {
      source[newAttribute] = source[attribute];
    }
    if (newAttribute !== attribute) {
      delete source[attribute];
    }
  });
  return source;
}

/**
 * 暴露给外部单独使用的方法
 * @param {object} source
 * @return {object}
 */
const snake = exports.snake = source => {
  if ((0, _lodash.isString)(source)) {
    return (0, _lodash.snakeCase)(source);
  } else if ((0, _lodash.isArray)(source)) {
    return source.map(item => (0, _lodash.snakeCase)(item));
  } else if ((0, _lodash.isObject)(source)) {
    const dest = {};
    Object.keys(source).forEach(item => {
      dest[(0, _lodash.snakeCase)(item)] = source[item];
    });
    return dest;
  }
  return source;
};

exports.default = Bookshelf => {
  const proto = Bookshelf.Model.prototype;

  Bookshelf.Model = Bookshelf.Model.extend({
    initialize(...args) {
      this.magicCase = true;
      // 下面代码对 new Model().save({mapId}) 方式新增记录生效
      this.on('saving', () => {
        const source = this.attributes;
        Object.keys(source).forEach(attribute => {
          const newAttribute = (0, _lodash.snakeCase)(attribute);
          if (newAttribute !== attribute) {
            source[newAttribute] = source[attribute];
            delete source[attribute];
          }
        });
        this.attributes = source;
      });

      return proto.initialize.apply(this, args);
    },

    // eslint-disable-next-line
    toJSON: function (options) {
      const attrs = proto.toJSON.call(this, options);
      return toCamelCase(attrs, this.constructor.jsonColumns);
    }
  });
};