'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _util = require('util');

var _koaRouter = require('koa-router');

var _koaRouter2 = _interopRequireDefault(_koaRouter);

var _koaCompose = require('koa-compose');

var _koaCompose2 = _interopRequireDefault(_koaCompose);

var _collection = require('bookshelf/lib/collection');

var _collection2 = _interopRequireDefault(_collection);

var _lodash = require('lodash');

var _magicCase = require('./magic-case');

var _responseEnvelope = require('./response-envelope');

var _responseEnvelope2 = _interopRequireDefault(_responseEnvelope);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

/**
 * @class
 */
class ResourceRouter extends _koaRouter2.default {

  /**
   * @static
   * @param {Object} options
   * @return {Router}
   */
  static define(options) {
    // eslint-disable-next-line
    let { collection, setup } = options,
        rest = _objectWithoutProperties(options, ['collection', 'setup']);
    if (options instanceof Function || options instanceof _collection2.default) {
      collection = options;
      options = undefined;
    }
    options = rest || options;
    setup = setup || (router => router.crud());
    const router = new this(collection, options);
    setup(router);
    return router;
  }

  /**
   * @constructor
   * @param {Collection} collection
   * @param {Object} options
   */
  // eslint-disable-next-line
  constructor(collection, options) {
    options = (0, _lodash.defaults)(options, {
      root: '',
      id: '',
      name: '',
      prefix: ''
    });
    super(options);
    this.methods = { create: false, read: false, update: false, destroy: false };
    this.prefix(options.prefix);
    this.collection = collection;
    if (!(0, _lodash.isFunction)(collection)) {
      options.model = options.model || collection.model;
      options.id = options.id || options.model.prototype.idAttribute;
      this.collection = () => collection;
    }
    options.name = options.name || options.model.prototype.tableName;
    options.root = options.root || `/${options.name}`;
    options.title = options.title || options.name;
    options.description = options.description || options.title;
    options.id = options.id || 'id';
    this.options = options;

    this.pattern = {
      root: options.root || '/',
      item: `${options.root ? options.root : ''}/:${options.id}`
    };
  }

  /**
   * Create a new record
   * @param {Object|Function} middlewave
   * @param {Object} options
   * @return {Object}
   */
  create(middlewave, options) {
    const { collection, pattern } = this;
    this.methods.create = true;

    let createMiddlewave = null;
    if ((0, _lodash.isFunction)(middlewave)) {
      createMiddlewave = middlewave;
    } else {
      createMiddlewave = (() => {
        var _ref = _asyncToGenerator(function* (ctx, next) {
          const attributes = ctx.state.attributes || (0, _magicCase.snake)(ctx.request.body);
          if (collection(ctx).relatedData) {
            ctx.state.resource = yield collection(ctx).create(attributes);
          } else {
            ctx.state.resource = collection(ctx).model.forge();
            yield ctx.state.resource.save(attributes);
          }
          yield next();
          ctx.status = 201;
          ctx.body = (0, _responseEnvelope2.default)({
            code: 0,
            message: 'Success',
            result: ctx.state.resource
          });
        });

        return function createMiddlewave(_x, _x2) {
          return _ref.apply(this, arguments);
        };
      })();
      options = middlewave;
    }

    const { beforeMiddlewaves, afterMiddlewaves } = options || {};
    const applyCreateMiddlewares = (0, _lodash.compact)([].concat(beforeMiddlewaves, createMiddlewave, afterMiddlewaves));
    this.post(pattern.root, (0, _koaCompose2.default)(applyCreateMiddlewares));
    return this;
  }

  /**
   * Read records
   * @param {Object} options
   * @return {Object}
   */
  read(options) {
    this.methods.read = true;
    const { collection, options: { id }, pattern } = this;

    // eslint-disable-next-line
    const listMiddleware = (() => {
      var _ref2 = _asyncToGenerator(function* (ctx, next) {
        const { embed, mask, page, pageSize, limit, offset } = ctx.query;
        const model = ctx.state.query || collection(ctx).model.forge();

        let { where, andWhere, orWhere, sort } = ctx.query;
        let fetchParams = { required: false };
        let result = {};

        if (model.magicCase) {
          if (sort) {
            if ((0, _lodash.startsWith)(sort, '-')) {
              sort = `-${(0, _magicCase.snake)(sort.substr(1, sort.length))}`;
            } else {
              sort = (0, _magicCase.snake)(sort);
            }
          }
          if (where) {
            where = (0, _magicCase.snake)(where);
          }
          if (andWhere) {
            andWhere = (0, _magicCase.snake)(andWhere);
          }
          if (orWhere) {
            orWhere = (0, _magicCase.snake)(orWhere);
          }
        }

        if (embed) {
          fetchParams.withRelated = embed.split(',');
        }

        if (where) {
          // where=id\&where=\>\&where=1
          if (Array.isArray(where)) {
            model.query(...['where'].concat(where));
          } else if ((0, _util.isObject)(where)) {
            // where[name]=sales
            model.where(where);
          }
        }

        if (orWhere) {
          if (Array.isArray(orWhere)) {
            (0, _lodash.chunk)(orWhere, 3).forEach(function (item) {
              // 确保拆分后的数组是一个完整的 where 条件
              if (item.length === 3) {
                model.query(...['orWhere'].concat(item));
              }
            });
          } else if ((0, _util.isObject)(orWhere)) {
            model.query({ orWhere });
          }
        }

        if (andWhere) {
          if (Array.isArray(andWhere)) {
            (0, _lodash.chunk)(andWhere, 3).forEach(function (item) {
              // 确保拆分后的数组是一个完整的 where 条件
              if (item.length === 3) {
                model.query(...['andWhere'].concat(item));
              }
            });
          } else if ((0, _util.isObject)(andWhere)) {
            model.query({ andWhere });
          }
        }

        // Order by support
        if (sort) {
          sort.split(',').forEach(function (field) {
            model.orderBy(field);
          });
        }

        // Pagination support
        if (page || pageSize || limit || offset) {
          if (page || pageSize) {
            fetchParams = _extends({
              page,
              pageSize
            }, fetchParams);
          } else {
            fetchParams = _extends({
              limit,
              offset
            }, fetchParams);
          }

          yield model.fetchPage(fetchParams).then(function (items) {
            if (items) {
              if (mask) {
                result = {
                  pagination: items.pagination,
                  list: items.mask(mask)
                };
              } else {
                result = {
                  pagination: items.pagination,
                  list: items.toJSON()
                };
              }
            }
          }).catch(function (e) {
            console.log(e);
          });
        } else {
          yield model.fetchAll(fetchParams).then(function (items) {
            if (items) {
              if (mask) {
                result = items.mask(mask);
              } else {
                result = items.toJSON();
              }
            }
          }).catch(function (e) {
            console.log(e);
          });
        }

        yield next();

        ctx.status = 200;
        ctx.body = (0, _responseEnvelope2.default)({
          code: 0,
          message: 'Success',
          result
        });
      });

      return function listMiddleware(_x3, _x4) {
        return _ref2.apply(this, arguments);
      };
    })();

    const itemMiddleware = (() => {
      var _ref3 = _asyncToGenerator(function* (ctx, next) {
        const { embed, mask } = ctx.query;
        const fetchParams = { required: true };
        if (embed) {
          fetchParams.withRelated = embed.split(',');
        }

        const item = yield collection(ctx).query(function (q) {
          return q.where({ [id]: ctx.params.id });
        }).fetchOne(fetchParams);

        if (item) {
          ctx.state.resource = mask ? item.mask(mask) : item.toJSON();
          yield next();
          ctx.body = (0, _responseEnvelope2.default)({
            code: 0,
            message: 'Success',
            result: ctx.state.resource
          });
          ctx.status = 200;
        } else {
          ctx.body = (0, _responseEnvelope2.default)({
            code: 404,
            message: 'Not found'
          });
        }
      });

      return function itemMiddleware(_x5, _x6) {
        return _ref3.apply(this, arguments);
      };
    })();

    const {
      beforeMiddlewaves,
      afterMiddlewaves,
      beforeListRouter,
      afterListRouter,
      beforeItemRouter,
      afterItemRouter
    } = options || {};
    const applyListMiddlewares = (0, _lodash.compact)([].concat(beforeListRouter, beforeMiddlewaves, listMiddleware, afterListRouter, afterMiddlewaves));
    const applyItemMiddlewares = (0, _lodash.compact)([].concat(beforeItemRouter, beforeMiddlewaves, itemMiddleware, afterItemRouter, afterMiddlewaves));

    // read list
    this.get(pattern.root, (0, _koaCompose2.default)(applyListMiddlewares));
    // read item
    this.get(pattern.item, (0, _koaCompose2.default)(applyItemMiddlewares));

    return this;
  }

  /**
   * Update record
   * @param {Object|Function} middlewave
   * @param {Object} options
   * @return {Object}
   */
  update(middlewave, options) {
    const { collection, options: { id }, pattern } = this;
    this.methods.update = true;
    let updateMiddleware = null;
    if ((0, _lodash.isFunction)(middlewave)) {
      updateMiddleware = middlewave;
    } else {
      updateMiddleware = (() => {
        var _ref4 = _asyncToGenerator(function* (ctx, next) {
          const attributes = ctx.state.attributes || (0, _magicCase.snake)(ctx.request.body);
          ctx.state.resource = (yield collection(ctx).query(function (q) {
            return q.where({ [id]: ctx.params[id] });
          }).fetch({ required: true })).first();
          if (ctx.state.resource) {
            const updatedResource = yield ctx.state.resource.save(attributes, {
              method: 'update',
              patch: true
            });
            yield next();
            ctx.body = (0, _responseEnvelope2.default)({
              code: 0,
              message: 'Success',
              result: updatedResource
            });
            ctx.status = 202;
          } else {
            ctx.body = (0, _responseEnvelope2.default)({
              code: 404,
              message: 'Not found'
            });
          }
        });

        return function updateMiddleware(_x7, _x8) {
          return _ref4.apply(this, arguments);
        };
      })();
      options = middlewave;
    }

    const { beforeMiddlewaves, afterMiddlewaves } = options || {};
    const applyUpdateMiddlewares = (0, _lodash.compact)([].concat(beforeMiddlewaves, updateMiddleware, afterMiddlewaves));
    this.put(pattern.item, (0, _koaCompose2.default)(applyUpdateMiddlewares));
    this.patch(pattern.item, (0, _koaCompose2.default)(applyUpdateMiddlewares));

    return this;
  }

  /**
   * Remove a record
   * @param {Object|Function} middlewave
   * @param {Object} options
   * @return {Object}
   */
  destroy(middlewave, options) {
    const { collection, pattern, options: { id } } = this;
    this.methods.destroy = true;

    let deleteMiddlewave = null;
    if ((0, _lodash.isFunction)(middlewave)) {
      deleteMiddlewave = middlewave;
    } else {
      deleteMiddlewave = (() => {
        var _ref5 = _asyncToGenerator(function* (ctx, next) {
          ctx.state.resource = yield collection(ctx).query(function (q) {
            return q.where({ [id]: ctx.params[id] });
          }).fetchOne({ require: false });
          if (ctx.state.resource) {
            ctx.state.deleted = ctx.state.resource.toJSON();
            yield ctx.state.resource.destroy();
            yield next();
            ctx.body = (0, _responseEnvelope2.default)({
              code: 0,
              message: 'Success'
            });
            /* ctx.status = 204; */
          } else {
            ctx.body = (0, _responseEnvelope2.default)({
              code: 404,
              message: 'Not found'
            });
          }
        });

        return function deleteMiddlewave(_x9, _x10) {
          return _ref5.apply(this, arguments);
        };
      })();
      options = middlewave;
    }

    const { beforeMiddlewaves, afterMiddlewaves } = options || {};
    const applyDeleteMiddlewares = (0, _lodash.compact)([].concat(beforeMiddlewaves, deleteMiddlewave, afterMiddlewaves));

    this.del(pattern.item, (0, _koaCompose2.default)(applyDeleteMiddlewares));
    return this;
  }

  /**
   * Auto generate CRUD
   * @method
   * @return {Object}
   */
  crud() {
    return this.create().read().update().destroy();
  }

  /**
   * Nest router
   */
  // children(...children) {
  //   const { pattern } = this;
  //   this.use(
  //     `${pattern}/:${this.foreignId}(${this.idType})`,
  //     async (ctx, next) => {
  //       ctx.state.nested = ctx.state.nested || {};
  //       ctx.state.nested[this.name] = await this.collection(ctx).query(q =>
  //         q.where({ [this.idAttribute]: ctx.params[this.foreignId] })
  //       ).fetchOne({ require: true });
  //       await next();
  //     },
  //     ...children.map( Child => Child instanceof Base
  //       ? Child.routes()
  //       : Child.prototype instanceof Base
  //         ? new Child().routes()
  //         : Child)
  //   )
  //   return this;
  // }
}
exports.default = ResourceRouter;